@@ -10,9 +10,10 @@ from importlib import reload
 from pathlib import Path
 from typing import Dict, Iterable, List, Optional, Set, Tuple, Union
 
-import pkg_resources
 import requests
 import spacy
+import thinc
+from pkg_resources import load_entry_point
 from requests import HTTPError
 from spacy.language import Language
 
@@ -24,6 +25,8 @@ from libratom.lib.pff import pff_msg_to_string
 
 logger = logging.getLogger(__name__)
 
+_cached_spacy_models = {}
+
 
 def get_ratom_settings() -> List[Tuple[str, Union[int, str]]]:
     return [
@@ -89,6 +92,19 @@ def get_spacy_models() -> Dict[str, List[str]]:
     return releases
 
 
+def get_cached_spacy_model(name: str) -> Optional[Language]:
+    """
+    Returns a cached preloaded spaCy model
+    """
+
+    try:
+        return _cached_spacy_models[name]
+    except KeyError:
+        model = _cached_spacy_models[name] = load_spacy_model(name)
+
+    return model
+
+
 def load_spacy_model(spacy_model_name: str) -> Optional[Language]:
     """
     Loads and returns a given spaCy model
@@ -117,8 +133,20 @@ def load_spacy_model(spacy_model_name: str) -> Optional[Language]:
                 logger.error(f"Unable to install spacy model {spacy_model_name}")
                 return None
 
-            # Now try loading it again
-            reload(pkg_resources)
+            # Specific steps for transformer models (very brittle and likely not permanent)
+            if spacy_model_name.endswith("_trf"):
+
+                # If we just installed a transformer model along with spacy-transformers in a child process
+                # we need to set up an entry point for spacy-transformers in the current process.
+                # This entry point will be registered as a pipeline factory function by the model's language class.
+                load_entry_point("spacy-transformers", "spacy_factories", "transformer")
+
+                # If Pytorch was also just installed, certain modules that depend on it
+                # may need reloading to work in the current process
+                reload(thinc.util)
+                reload(thinc.shims.pytorch)
+
+            # Now try loading the model again
             spacy_model = spacy.load(spacy_model_name)
 
         else:
