@@ -4,21 +4,24 @@ Set of utility functions that use spaCy to perform named entity recognition
 """
 
 import logging
-import multiprocessing
 from datetime import datetime
+from multiprocessing import Pool, current_process
 from pathlib import Path
 from typing import Callable, Dict, Iterable, List, Optional, Tuple
 
-from spacy.language import Language
 from sqlalchemy.orm.session import Session
 
 from libratom.lib.base import AttachmentMetadata
 from libratom.lib.concurrency import get_messages, imap_job, worker_init
-from libratom.lib.core import (
+from libratom.lib.constants import (
     RATOM_DB_COMMIT_BATCH_SIZE,
     RATOM_MSG_BATCH_SIZE,
     RATOM_MSG_PROGRESS_STEP,
+    RATOM_SPACY_MODEL_MAX_LENGTH,
+    BodyType,
 )
+from libratom.lib.core import get_cached_spacy_model
+from libratom.lib.utils import cleanup_message_body
 from libratom.models import Attachment, Entity, FileReport, Message
 
 logger = logging.getLogger(__name__)
@@ -28,10 +31,13 @@ logger = logging.getLogger(__name__)
 def process_message(
     filepath: str,
     message_id: int,
-    message: str,
+    body: str,
+    body_type: BodyType,
     date: datetime,
     attachments: List[AttachmentMetadata],
-    spacy_model: Language,
+    spacy_model_name: str,
+    headers: Optional[str] = None,
+    include_message_contents: bool = False,
 ) -> Tuple[Dict, Optional[str]]:
     """
     Job function for the worker processes
@@ -48,11 +54,30 @@ def process_message(
 
     try:
         # Extract entities from the message
-        doc = spacy_model(message)
+        message_body = cleanup_message_body(
+            body, body_type, RATOM_SPACY_MODEL_MAX_LENGTH
+        )
+
+        # https://github.com/explosion/spaCy/issues/6662#issuecomment-753889021
+        if (
+            spacy_model_name.endswith("_trf")
+            and current_process().name != "MainProcess"
+        ):
+            # pylint: disable=all
+            import torch
+
+            torch.set_num_threads(1)
+
+        spacy_model = get_cached_spacy_model(spacy_model_name)
+        doc = spacy_model(message_body)
         res["entities"] = [(ent.text, ent.label_) for ent in doc.ents]
 
         res["processing_end_time"] = datetime.utcnow()
 
+        if include_message_contents:
+            res["body"] = message_body
+            res["headers"] = headers
+
         return res, None
 
     except Exception as exc:
@@ -62,7 +87,8 @@ def process_message(
 def extract_entities(
     files: Iterable[Path],
     session: Session,
-    spacy_model: Language,
+    spacy_model_name: str,
+    include_message_contents: bool = False,
     jobs: int = None,
     processing_progress_callback: Callable = None,
     reporting_progress_callback: Callable = None,
@@ -87,7 +113,7 @@ def extract_entities(
     _file_reports = session.query(FileReport).all()  # noqa: F841
 
     # Start of multiprocessing
-    with multiprocessing.Pool(processes=jobs, initializer=worker_init) as pool:
+    with Pool(processes=jobs, initializer=worker_init) as pool:
 
         logger.debug(f"Starting pool with {pool._processes} processes")
 
@@ -95,14 +121,15 @@ def extract_entities(
         msg_count = 0
 
         try:
-
             for msg_count, worker_output in enumerate(
                 pool.imap_unordered(
                     process_message,
                     get_messages(
                         files,
-                        spacy_model=spacy_model,
+                        spacy_model_name=spacy_model_name,
                         progress_callback=processing_update_progress,
+                        include_message_contents=include_message_contents,
+                        with_headers=include_message_contents,
                         **kwargs,
                     ),
                     chunksize=RATOM_MSG_BATCH_SIZE,
